@@ -3,12 +3,15 @@ package Business::ES::NIF;
 # ABSTRACT: Validate Spanish NIF, NIE and CIF numbers
 
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use strict;
 use warnings;
 
 
+use Class::XSAccessor { accessors => [qw(error extra nif status type vies_check vies_error)] };
+
+
 use constant {
 	      NIF_LETRAS => 'TRWAGMYFPDXBNJZSQVHLCKE',
 	      CIF_LETRAS => 'JABCDEFGHI',
@@ -19,133 +22,124 @@ use constant {
 	      RE_NIE     => qr/^([XYZ])([0-9]{7})([A-Z])$/
 	     };
 
-
-sub type  { return shift->{type};  }
-sub extra { return shift->{extra}; }
-sub error { return shift->{error}; }
+my %PROVINCIAS = (
+		  '00' => 'No Residente',
+		  '01' => 'Álava',
+		  '02' => 'Albacete', 
+		  '03' => 'Alicante',
+		  '04' => 'Almería',
+		  '05' => 'Ávila',
+		  '06' => 'Badajoz',
+		  '07' => 'Islas Baleares',
+		  '08' => 'Barcelona',
+		  '09' => 'Burgos',
+		  '10' => 'Cáceres',
+		  '11' => 'Cádiz',
+		  '12' => 'Castellón',
+		  '13' => 'Ciudad Real',
+		  '14' => 'Córdoba',
+		  '15' => 'La Coruña',
+		  '16' => 'Cuenca',
+		  '17' => 'Gerona',
+		  '18' => 'Granada',
+		  '19' => 'Guadalajara',
+		  '20' => 'Guipúzcoa',
+		  '21' => 'Huelva',
+		  '22' => 'Huesca',
+		  '23' => 'Jaén',
+		  '24' => 'León',
+		  '25' => 'Lérida',
+		  '26' => 'La Rioja',
+		  '27' => 'Lugo',
+		  '28' => 'Madrid',
+		  '29' => 'Málaga',
+		  '30' => 'Murcia',
+		  '31' => 'Navarra',
+		  '32' => 'Orense',
+		  '33' => 'Asturias',
+		  '34' => 'Palencia',
+		  '35' => 'Las Palmas',
+		  '36' => 'Pontevedra',
+		  '37' => 'Salamanca',
+		  '38' => 'Santa Cruz de Tenerife',
+		  '39' => 'Cantabria',
+		  '40' => 'Segovia',
+		  '41' => 'Sevilla',
+		  '42' => 'Soria',
+		  '43' => 'Tarragona',
+		  '44' => 'Teruel',
+		  '45' => 'Toledo',
+		  '46' => 'Valencia',
+		  '47' => 'Valladolid',
+		  '48' => 'Vizcaya',
+		  '49' => 'Zamora',
+		  '50' => 'Zaragoza',
+		  '51' => 'Ceuta',
+		  '52' => 'Melilla',
+		  '53' => 'Alicante',         
+		  '54' => 'Alicante',         
+		  '55' => 'Gerona',           
+		  '56' => 'Córdoba',          
+		  '57' => 'Islas Baleares',   
+		  '58' => 'Barcelona',        
+		  '59' => 'Barcelona',
+		  '60' => 'Barcelona',
+		  '61' => 'Barcelona',
+		  '62' => 'Barcelona',
+		  '63' => 'Barcelona',
+		  '64' => 'Barcelona',
+		  '65' => 'Barcelona',
+		  '66' => 'Barcelona',
+		  '67' => 'Gerona',           
+		  '68' => 'Barcelona',        
+		  '70' => 'La Coruña',        
+		  '71' => 'Guipúzcoa',        
+		  '72' => 'Cádiz',            
+		  '73' => 'Murcia',           
+		  '74' => 'Asturias',         
+		  '75' => 'Las Palmas',       
+		  '76' => 'Santa Cruz de Tenerife',
+		  '77' => 'Tarragona',        
+		  '78' => 'Madrid',           
+		  '79' => 'Madrid',
+		  '80' => 'Madrid',
+		  '81' => 'Madrid',
+		  '82' => 'Madrid',
+		  '83' => 'Madrid',
+		  '84' => 'Madrid',
+		  '85' => 'Madrid',
+		  '86' => 'Madrid',
+		  '87' => 'Madrid',
+		  '88' => 'Madrid',
+		  '90' => 'Sevilla',          
+		  '91' => 'Sevilla',          
+		  '92' => 'Málaga',           
+		  '93' => 'Málaga',           
+		  '94' => 'Pontevedra',       
+		  '95' => 'Vizcaya',          
+		  '96' => 'Valencia',         
+		  '97' => 'Valencia',
+		  '98' => 'Valencia',
+		  '99' => 'Zaragoza',
+		 );
 
 
-sub is_nie  { return shift->{type} eq 'NIE'  }
-sub is_nif  { return shift->{type} eq 'NIF'  }
-sub is_nife { return shift->{type} eq 'NIFe' }
-sub is_cif  { return shift->{type} eq 'CIF'  }
-sub is_cife { return shift->{type} eq 'CIFe' }
+
+sub is_nie  { return shift->type eq 'NIE'  }
+sub is_nif  { return shift->type eq 'NIF'  }
+sub is_nife { return shift->type eq 'NIFe' }
+sub is_cif  { return shift->type eq 'CIF'  }
+sub is_cife { return shift->type eq 'CIFe' }
 
 sub _provincia {
-  my $provincia = shift;
-  
-  my %PROVINCIAS = (
-		    '00' => 'No Residente',
-		    '01' => 'Álava',
-		    '02' => 'Albacete', 
-		    '03' => 'Alicante',
-		    '04' => 'Almería',
-		    '05' => 'Ávila',
-		    '06' => 'Badajoz',
-		    '07' => 'Islas Baleares',
-		    '08' => 'Barcelona',
-		    '09' => 'Burgos',
-		    '10' => 'Cáceres',
-		    '11' => 'Cádiz',
-		    '12' => 'Castellón',
-		    '13' => 'Ciudad Real',
-		    '14' => 'Córdoba',
-		    '15' => 'La Coruña',
-		    '16' => 'Cuenca',
-		    '17' => 'Gerona',
-		    '18' => 'Granada',
-		    '19' => 'Guadalajara',
-		    '20' => 'Guipúzcoa',
-		    '21' => 'Huelva',
-		    '22' => 'Huesca',
-		    '23' => 'Jaén',
-		    '24' => 'León',
-		    '25' => 'Lérida',
-		    '26' => 'La Rioja',
-		    '27' => 'Lugo',
-		    '28' => 'Madrid',
-		    '29' => 'Málaga',
-		    '30' => 'Murcia',
-		    '31' => 'Navarra',
-		    '32' => 'Orense',
-		    '33' => 'Asturias',
-		    '34' => 'Palencia',
-		    '35' => 'Las Palmas',
-		    '36' => 'Pontevedra',
-		    '37' => 'Salamanca',
-		    '38' => 'Santa Cruz de Tenerife',
-		    '39' => 'Cantabria',
-		    '40' => 'Segovia',
-		    '41' => 'Sevilla',
-		    '42' => 'Soria',
-		    '43' => 'Tarragona',
-		    '44' => 'Teruel',
-		    '45' => 'Toledo',
-		    '46' => 'Valencia',
-		    '47' => 'Valladolid',
-		    '48' => 'Vizcaya',
-		    '49' => 'Zamora',
-		    '50' => 'Zaragoza',
-		    '51' => 'Ceuta',
-		    '52' => 'Melilla',
-		    '53' => 'Alicante',         
-		    '54' => 'Alicante',         
-		    '55' => 'Gerona',           
-		    '56' => 'Córdoba',          
-		    '57' => 'Islas Baleares',   
-		    '58' => 'Barcelona',        
-		    '59' => 'Barcelona',
-		    '60' => 'Barcelona',
-		    '61' => 'Barcelona',
-		    '62' => 'Barcelona',
-		    '63' => 'Barcelona',
-		    '64' => 'Barcelona',
-		    '65' => 'Barcelona',
-		    '66' => 'Barcelona',
-		    '67' => 'Gerona',           
-		    '68' => 'Barcelona',        
-		    '70' => 'La Coruña',        
-		    '71' => 'Guipúzcoa',        
-		    '72' => 'Cádiz',            
-		    '73' => 'Murcia',           
-		    '74' => 'Asturias',         
-		    '75' => 'Las Palmas',       
-		    '76' => 'Santa Cruz de Tenerife',
-		    '77' => 'Tarragona',        
-		    '78' => 'Madrid',           
-		    '79' => 'Madrid',
-		    '80' => 'Madrid',
-		    '81' => 'Madrid',
-		    '82' => 'Madrid',
-		    '83' => 'Madrid',
-		    '84' => 'Madrid',
-		    '85' => 'Madrid',
-		    '86' => 'Madrid',
-		    '87' => 'Madrid',
-		    '88' => 'Madrid',
-		    '90' => 'Sevilla',          
-		    '91' => 'Sevilla',          
-		    '92' => 'Málaga',           
-		    '93' => 'Málaga',           
-		    '94' => 'Pontevedra',       
-		    '95' => 'Vizcaya',          
-		    '96' => 'Valencia',         
-		    '97' => 'Valencia',
-		    '98' => 'Valencia',
-		    '99' => 'Zaragoza',
-		   );
-  
-  return unless $provincia =~ /^[0-9]{2}$/;
-  
-  return $PROVINCIAS{$provincia};
+  my $codigo = shift;
+
+  return $PROVINCIAS{$codigo} if $codigo =~ /^[0-9]{2}$/;
+  return undef;
 }
 
-
 sub _check_cif {
-  my $self = shift;
-  my $sociedad = shift;
-  my $inscripcion = shift;
-  my $control = shift;
+  my ($sociedad,$inscripcion,$control) = @_;
   
   my @n = split //, $inscripcion;
   my $pares = $n[1] + $n[3] + $n[5];
@@ -171,132 +165,88 @@ sub _check_cif {
 }
 
 sub _validate_cif {
-  my $self = shift;
-  my $cif  = shift;
+  my ($self,$sociedad, $inscripcion, $control) = @_;
 
-  $self->{type} = 'CIF';
-  
-  $cif =~ RE_CIF;
+  $self->status(_check_cif($sociedad, $inscripcion, $control));
+  $self->{provincia} = _provincia(substr($inscripcion,0,2));
 
-  my $status = $self->_check_cif($1, $2, $3);
-
-  $self->{provincia} = _provincia(substr($cif,1,2));
-  
-  return 0 unless $status;
-  
-  $self->{extra} = {
-                    'A' => 'Sociedad Anonima - S.A',
-                    'B' => 'Sociedad Limitada - S.L',
-                    'C' => 'Sociedad Colectiva - S.C',
-                    'D' => 'Sociedades comanditarias',
-                    'E' => 'Comunidad de bienes y herencias',
-                    'F' => 'Sociedades cooperativas',
-                    'G' => 'Asociaciones',
-                    'H' => 'Comunidaddes de propietarios',
-                    'J' => 'Sociedades civiles',
-                    'P' => 'Corporaciones locales',
-                    'Q' => 'Organismos publicos',
-                    'N' => 'Entidades extranjeras',
-                    'R' => 'Congregaciones e instituciones religiosas',
-                    'U' => 'Uniones temporales de epresas',
-                    'V' => 'Otros tipos de sociedades',
-                    'W' => 'Establecimientos permanentes de entidades no residentes en España',
-                   }->{ substr($cif, 0, 1) };
-
-  return 1;
+  $self->type('CIF');
+  $self->extra({
+		'A' => 'Sociedad Anonima - S.A',
+		'B' => 'Sociedad Limitada - S.L',
+		'C' => 'Sociedad Colectiva - S.C',
+		'D' => 'Sociedades comanditarias',
+		'E' => 'Comunidad de bienes y herencias',
+		'F' => 'Sociedades cooperativas',
+		'G' => 'Asociaciones',
+		'H' => 'Comunidaddes de propietarios',
+		'J' => 'Sociedades civiles',
+		'P' => 'Corporaciones locales',
+		'Q' => 'Organismos publicos',
+		'N' => 'Entidades extranjeras',
+		'R' => 'Congregaciones e instituciones religiosas',
+		'U' => 'Uniones temporales de epresas',
+		'V' => 'Otros tipos de sociedades',
+		'W' => 'Establecimientos permanentes de entidades no residentes en España',
+	       }->{ $sociedad } );
 }
 
 sub _validate_cife {
-  my $self = shift;
-  my $cif  = shift;
+  my ($self,$sociedad, $inscripcion, $control) = @_;
 
-  $self->{type} = 'CIFe';
-  
-  $cif =~ RE_CIFe;
+  $self->status(_check_cif($sociedad, $inscripcion, $control));
 
-  my $status = $self->_check_cif($1,$2,$3);
-  
-  return 0 unless $status;
-  
-  $self->{extra} = {
-		    'S' => 'Órganos de la Administración del Estado',
-		    'Q' => 'Organismos autónomos o instituciones religiosas',
-		    'P' => 'Corporaciones locales',
-		    'K' => 'Formato antiguo (DNI sin letra de control)',
-		   }->{ substr($cif, 0, 1) };
+  $self->type('CIFe');
+  $self->extra({
+	       'S' => 'Órganos de la Administración del Estado',
+	       'Q' => 'Organismos autónomos o instituciones religiosas',
+	       'P' => 'Corporaciones locales',
+	       'K' => 'Formato antiguo (DNI sin letra de control)',
+	      }->{ $sociedad });
+}
 
-  return 1;
+sub _check_nif {
+  my ($NIF,$DC) = @_;
+
+  return substr( NIF_LETRAS , $NIF % 23, 1) eq $DC ? 1 : 0;
 }
 
 sub _validate_nif {
   my $self = shift;
-  my $nif  = shift;
 
-  $self->{type} = 'NIF';
-  
-  $nif =~ RE_NIF;
+  $self->status(_check_nif(shift,shift));
 
-  my $status = substr( NIF_LETRAS , $1 % 23, 1) eq $2 ? 1 : 0;
-
-  return 0 unless $status;
-
-  $self->{extra} = 'NIF';
-  
-  return 1;
+  $self->type('NIF');
+  $self->extra('NIF');
 }
+
 sub _validate_nife {
-  my $self = shift;
-  my $nif  = shift;
+  my $self = shift; 
 
-  $self->{type} = 'NIFe';
-  
-  $nif =~ /^[KLM]([0-9]{7})([A-Z])$/;
+  $self->status(_check_nif(shift,shift));
 
-  my $status = substr(NIF_LETRAS, $1 % 23, 1) eq $2 ? 1 : 0;
-
-  return 0 unless $status;
-
-  $self->{extra} = 'NIF Especial (KLM)';
-  
-  return 1;
+  $self->type('NIFe'); 
+  $self->extra('NIF Especial (KLM)');
 }
 
 sub _validate_nie {
-  my $self = shift;
-  my $nie  = shift;
+  my ($self,$NIE,$NIF,$DC) = @_;
 
-  $self->{type} = 'NIE';
-  
-  $nie =~ RE_NIE;
+  if    ($NIE eq 'X') { $NIF = '0'.$NIF; }
+  elsif ($NIE eq 'Y') { $NIF = '1'.$NIF; }
+  elsif ($NIE eq 'Z') { $NIF = '2'.$NIF; }
 
-  my ($NIE,$NIF,$DC) = ($1,$2,$3);
-                       
-  for ($NIE) {
-    $NIF = '0'.$NIF if /X/;
-    $NIF = '1'.$NIF if /Y/;
-    $NIF = '2'.$NIF if /Z/;
-  }
-  
-  my $status = substr( NIF_LETRAS , $NIF % 23, 1) eq $DC ? 1 : 0;
+  $self->status(_check_nif($NIF,$DC));
 
-  return 0 unless $status;
-
-  $self->{extra} = 'NIE';
-
-  return 1;
+  $self->type('NIE');
+  $self->extra('NIE');
 }
 
 
 sub new {
   my ($class, %args) = @_;
 
-  my $self = {
-	      nif    => undef,
-	      extra  => undef,
-	      status => 0,
-	      type   => undef,
-	      vies   => $args{vies} // 0
-	     };
+  my $self = { vies   => $args{vies} // 0 };
   
   $self = bless $self, $class;
 
@@ -314,55 +264,57 @@ sub set {
   $self->{vies} = 1 if $vies;
 
   unless (defined $nif) {
-    $self->{status} = 0;
-    $self->{error}  = "NIF vacío";
+    $self->status(0);
+    $self->error("NIF vacío");
     return;
   }
 
   $nif =~ tr/-. //d;
-  $self->{nif} = uc($nif);
 
+  $self->nif(uc($nif));
   $self->check;
 }
 
 
-sub is_valid { return shift->{status} ? 1 : 0; }
+sub is_valid { return shift->status ? 1 : 0; }
 
 
 sub check {
   my $self = shift;
-  my $nif  = $self->{nif};
+  my $nif  = $self->nif;
 
-  if    ($nif =~ RE_NIF)  { $self->{status} = $self->_validate_nif($nif);  }
-  elsif ($nif =~ RE_CIF)  { $self->{status} = $self->_validate_cif($nif);  }
-  elsif ($nif =~ RE_NIE)  { $self->{status} = $self->_validate_nie($nif);  }
-  elsif ($nif =~ RE_NIFe) { $self->{status} = $self->_validate_nife($nif); }
-  elsif ($nif =~ RE_CIFe) { $self->{status} = $self->_validate_cife($nif); }
+  if    ($nif =~ RE_NIF)  { $self->_validate_nif ($1, $2);     }
+  elsif ($nif =~ RE_CIF)  { $self->_validate_cif ($1, $2, $3); }
+  elsif ($nif =~ RE_NIE)  { $self->_validate_nie ($1, $2, $3); }
+  elsif ($nif =~ RE_NIFe) { $self->_validate_nife($2, $3);     }
+  elsif ($nif =~ RE_CIFe) { $self->_validate_cife($1, $2, $3); }
 
-  if ( $self->{status} ) {
+  if ( $self->status ) {
     $self->vies() if $self->{vies};
     return;
   }
   
-  $self->{error} = "Error formato de NIF/CIF/NIE";
-
+  $self->error("Error formato de NIF/CIF/NIE");
+  $self->status(0);
+  
   return;
 }
 
 
 sub vies {
   my $self = shift;
+  my $nif  = $self->nif;
   
-  unless ($self->{status}) {
-    $self->{vies_error} = "No se puede validar VIES: NIF no es válido";
-    $self->{vies_check} = 0;
-    return 0;
+  unless ($self->status) {
+    $self->vies_error("No se puede validar VIES: NIF no es válido");
+    $self->vies_check(0);
+    return;
   }
   
-  unless (defined $self->{nif} && length($self->{nif}) > 0) {
-    $self->{vies_error} = "No hay NIF para validar en VIES";
-    $self->{vies_check} = 0;
-    return 0;
+  unless (defined $nif && length($nif) > 0) {
+    $self->vies_error("No hay NIF para validar en VIES");
+    $self->vies_check(0);
+    return;
   }
   
   eval {
@@ -374,19 +326,18 @@ sub vies {
       die "Error Business::Tax::VAT::Validation";
     }
     
-    $self->{vies_check} = $vat->check('ES' . $self->{nif});
+    $self->vies_check($vat->check('ES' . $nif));
     
-    unless ($self->{vies_check}) {
-      $self->{vies_error} = $vat->get_last_error() // 'Error desconocido en VIES';
+    unless ($self->vies_check) {
+      $self->vies_error( $vat->get_last_error() // 'Error desconocido en VIES');
     }
   };
   
   if ($@) {
-    $self->{vies_error} = "Error cargando validación VIES: $@";
-    $self->{vies_check} = 0;
+    $self->vies_error("Error cargando validación VIES: $@");
+    $self->vies_check(0);
   }
-  
-  return $self->{vies_check} // 0;
+
 }
 
 
@@ -404,7 +355,7 @@ Business::ES::NIF - Validate Spanish NIF, NIE and CIF numbers
 
 =head1 VERSION
 
-version 0.11
+version 0.12
 
 =head1 SYNOPSIS
 
@@ -415,10 +366,11 @@ version 0.11
     $NIF->set('B01234567');
     $NIF->set('B01234567',1); <= Check VIES ( Business::Tax::VAT::Validation )
 
+    $NIF->status();
+
  Dump:
 
     bless( {
-           'status' => 1,
            'nif' => '01234567L',
            'vies' => 0,
            'extra' => 'NIF',
@@ -426,7 +378,6 @@ version 0.11
            }, 'NIF' );
 
     bless( {
-           'status' => 0,
            'nif' => 'B01234567',
            'vies' => 1,
            'vies_check' => 0,
@@ -451,6 +402,14 @@ Se puede activar la comprobacion sobre el VIES ( Business::Tax::VAT::Validation
 
 =head1 SUBROUTINES/METHODS
 
+=head2 nif
+
+=head2 status
+
+=head2 vies_check
+
+=head2 vies_error
+
 =head2 type
 
 Devuelve el tipo (NIF, NIE, CIF, CIFe, NIFe)
